using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Microsoft.Win32;
using System.Security.Principal;
using System.Diagnostics;

namespace UACDemo
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        private void OnFormLoad(object sender, EventArgs e)
        {
            AppDomain.CurrentDomain.SetPrincipalPolicy(
                PrincipalPolicy.WindowsPrincipal);
            txtUser.Text = System.Threading.Thread.CurrentPrincipal.Identity.Name;
            InteropDefs.SetButtonShield(button1, true);
        }

        private void OnWriteToProgramFiles(object sender, EventArgs e)
        {
            try
            {
                string folderPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles), "UACDemo");

                if (!Directory.Exists(folderPath))
                {
                    Directory.CreateDirectory(folderPath);
                }

                string filePath = Path.Combine(folderPath, "log.txt");

                using (TextWriter tw = new StreamWriter(filePath, true))
                {
                    tw.WriteLine("entry added " + DateTime.Now);
                }

                MessageBox.Show("Entry has successfully been added to Log.txt", "Write To Program Files", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (System.Security.SecurityException secEx)
            {
                MessageBox.Show(string.Format("SecurityException: {0}", secEx.Message), "Write To Program Files", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (UnauthorizedAccessException authEx)
            {
                MessageBox.Show(string.Format("UnauthorizedAccessException: {0}", authEx.Message), "Write To Program Files", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Write To Program Files", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void OnWriteToHKLM(object sender, EventArgs e)
        {
            try
            {
                using (RegistryKey regKey = Registry.LocalMachine.CreateSubKey(
                    @"Software\UACDemo", RegistryKeyPermissionCheck.ReadWriteSubTree))
                {

                    regKey.SetValue("SampleValue", "Updated at " + DateTime.Now);
                }

                MessageBox.Show("Registry key has successfully been updated", "Write To Registry HKLM", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (System.Security.SecurityException secEx)
            {
                MessageBox.Show(string.Format("SecurityException: {0}", secEx.Message), "Write To Registry HKLM", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (UnauthorizedAccessException authEx)
            {
                MessageBox.Show(string.Format("UnauthorizedAccessException: {0}", authEx.Message), "Write To Registry HKLM", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Write To Registry HKLM", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void OnIsAdministrator(object sender, EventArgs e)
        {
            WindowsIdentity wi = WindowsIdentity.GetCurrent();
            WindowsPrincipal wp = new WindowsPrincipal(wi);

            if (wp.IsInRole("Administrators"))
            {
                MessageBox.Show("Yes, you are an Administrator.", "Is Administrator?", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("No, you are not an Administrator.", "Is Administrator?", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void OnLaunchPrivilegedExecutable(object sender, EventArgs e)
        {
            try
            {
                string exePath = GetPrivilegedExePath();

                Process proc = new Process();
                proc.StartInfo = new ProcessStartInfo();
                proc.StartInfo.FileName = exePath; 
                proc.StartInfo.CreateNoWindow = true;
                proc.StartInfo.UseShellExecute = true;

                proc.EnableRaisingEvents = true;
                proc.Exited += new EventHandler(OnPrivilegedExeExited);
                proc.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        private void OnPrivilegedExeExited(object sender, EventArgs e)
        {
            Process proc = sender as Process;

            if (proc.ExitCode == 0)
            {
                MessageBox.Show("PrivilegedExe has been run successfully", "Write To Program Files", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show(String.Format("PrivilegedExe has returned with error code: {0}", proc.ExitCode),
                    "Write To Program Files", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private string GetPrivilegedExePath()
        {
            // Determine path of executable to launch
            // First, look in same folder
            string folder = Path.GetDirectoryName(
                System.Reflection.Assembly.GetEntryAssembly().Location);
            string exePath = Path.Combine(folder, "PrivilegedExe.exe");

            // If not found in same folder as calling app, 
            // assume it's in a sister VS project in the same solution
            if (!File.Exists(exePath))
            {
                // Go up three levels in the file system
                folder = Path.GetDirectoryName(folder);
                folder = Path.GetDirectoryName(folder);
                folder = Path.GetDirectoryName(folder);

                // Go to output folder
                folder = Path.Combine(folder, @"PrivilegedExe\bin\debug");
                exePath = Path.Combine(folder, "PrivilegedExe.exe");
            }

            return (exePath);
        }
    }
}